import { world, ItemStack } from "@minecraft/server";

function getRandomItemForBlock(blockId) {
    const items = {
        "minecraft:gold_ore": [{ id: "minecraft:tnt", count: 1 }],
        "minecraft:iron_ore": [
            { id: "minecraft:leather_helmet", count: 1 },
            { id: "minecraft:leather_chestplate", count: 1 },
            { id: "minecraft:leather_leggings", count: 1 },
            { id: "minecraft:leather_boots", count: 1 },
            { id: "minecraft:iron_helmet", count: 1 },
            { id: "minecraft:iron_chestplate", count: 1 },
            { id: "minecraft:iron_leggings", count: 1 },
            { id: "minecraft:iron_boots", count: 1 },
            { id: "minecraft:diamond_helmet", count: 1, chance: 0.04 },
            { id: "minecraft:diamond_chestplate", count: 1, chance: 0.04 },
            { id: "minecraft:diamond_leggings", count: 1, chance: 0.04 },
            { id: "minecraft:diamond_boots", count: 1, chance: 0.04 },
            { id: "minecraft:golden_helmet", count: 1 },
            { id: "minecraft:golden_chestplate", count: 1 },
            { id: "minecraft:golden_leggings", count: 1 },
            { id: "minecraft:golden_boots", count: 1 },
            { id: "minecraft:chainmail_helmet", count: 1 },
            { id: "minecraft:chainmail_chestplate", count: 1 },
            { id: "minecraft:chainmail_leggings", count: 1 },
            { id: "minecraft:chainmail_boots", count: 1 }
        ],
        "minecraft:diamond_ore": [
            { id: "minecraft:wooden_sword", count: 1 },
            { id: "minecraft:iron_sword", count: 1 },
            { id: "minecraft:diamond_sword", count: 1, chance: 0.03 },
            { id: "minecraft:stone_sword", count: 1 },
            { id: "minecraft:golden_sword", count: 1 }
        ],
        "minecraft:lapis_ore": [
            { id: "minecraft:golden_apple", count: 1 },
            { id: "minecraft:apple", count: 1 }
        ],
        "minecraft:stone": [
            { id: "minecraft:stone", count: 1, chance: 0.1 },
            { id: "minecraft:cobblestone", count: 1, chance: 0.5 }
        ]
    };
    
    const possibleItems = items[blockId];
    if (!possibleItems) return null;
    
    const randomChance = Math.random();
    let cumulativeChance = 0;

    for (const item of possibleItems) {
        cumulativeChance += item.chance || 1 / possibleItems.length;
        if (randomChance < cumulativeChance) {
            return item;
        }
    }

    return null;
}

world.afterEvents.playerBreakBlock.subscribe(ev => {
    const player = ev.player;
    
    if (!player.hasTag('game')) return;
    
    const block = ev.brokenBlockPermutation.type;
    const randomItem = getRandomItemForBlock(block.id);
    
    if (randomItem) {
        const item = new ItemStack(randomItem.id, randomItem.count);
        player.getComponent("minecraft:inventory").container.addItem(item);
    }
});
