import { world, system, ItemStack } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";

// Переменные
let checkWinnerInterval = null;
let gameTimer = null;
let gameDuration = 480;
const playerDeathCoords = {};

// Запуск игры
function startGame(duration) {
    world.getDimension('overworld').runCommandAsync(`tellraw @a {"rawtext":[{"text":"§aLoading map..."}]}`);

    system.runTimeout(() => {
        world.getDimension('overworld').runCommandAsync(`structure load airmap1 35 66 35 0_degrees none false true false`);
        world.getDimension('overworld').runCommandAsync(`structure load stonemap1 35 35 35 0_degrees none false true false`);
        world.getDimension('overworld').runCommandAsync(`structure load goldmap1 35 35 35 0_degrees none false true false 3`); //3
        world.getDimension('overworld').runCommandAsync(`structure load ironmap1 35 35 35 0_degrees none false true false 1.3`); //1.3
        world.getDimension('overworld').runCommandAsync(`structure load diamondmap1 35 35 35 0_degrees none false true false 1.5`); //1.5
        world.getDimension('overworld').runCommandAsync(`structure load lapismap1 35 35 35 0_degrees none false true false 1.6`); //1.6
        world.getDimension('overworld').runCommandAsync(`scoreboard objectives add player dummy`);
        world.getDimension('overworld').runCommandAsync(`scoreboard players set @a player 0`);

        // Установка командного блока
        const players = world.getPlayers();
        const playerPositions = [
            { x: 39, y: 51, z: 61 },
            { x: 61, y: 51, z: 39 },
            { x: 39, y: 51, z: 39 },
            { x: 61, y: 51, z: 61 }
        ];

        if (players.length > 0) {
            players.forEach((player, index) => {
                if (index < playerPositions.length) {
                    world.getDimension('overworld').runCommandAsync(`scoreboard players set ${player.name} player ${index + 1}`);
                    player.teleport(playerPositions[index]);
                }
            });
        }

        // Запуск игры
        system.runTimeout(() => {
            world.getDimension('overworld').runCommandAsync(`tag @a add game`);
            world.getDimension('overworld').runCommandAsync(`titleraw @a title {"rawtext":[{"text":"3"}]}`);
            world.getDimension('overworld').runCommandAsync(`clear @a`);
            world.getDimension('overworld').runCommandAsync(`playsound random.orb @a`);
            
            system.runTimeout(() => {
                world.getDimension('overworld').runCommandAsync(`titleraw @a title {"rawtext":[{"text":"2"}]}`);
                world.getDimension('overworld').runCommandAsync(`execute as @a[tag=game] at @s[tag=game] run fill ~1 ~ ~1 ~-1 ~2 ~-1 air`);
                world.getDimension('overworld').runCommandAsync(`playsound random.orb @a`);
                world.getDimension('overworld').runCommandAsync(`effect @a saturation 5 255 true`);
                world.getDimension('overworld').runCommandAsync(`effect @a regeneration 5 255 true`);                
                system.runTimeout(() => {
                    world.getDimension('overworld').runCommandAsync(`titleraw @a title {"rawtext":[{"text":"1"}]}`);
                    world.getDimension('overworld').runCommandAsync(`playsound random.orb @a`);
                    system.runTimeout(() => {
                        world.getDimension('overworld').runCommandAsync(`titleraw @a title {"rawtext":[{"text":"Go!"}]}`);
                        world.getDimension('overworld').runCommandAsync(`playsound random.levelup @a`);
                        world.getDimension('overworld').runCommandAsync(`replaceitem entity @a slot.weapon.mainhand 0 iron_pickaxe 1`);
                        world.getDimension('overworld').runCommandAsync(`enchant @a efficiency 3`);
                        world.getDimension('overworld').runCommandAsync(`enchant @a unbreaking 3`);
                        world.getDimension('overworld').runCommandAsync(`scoreboard players set @a player 0`);
                        world.getDimension('overworld').runCommandAsync(`gamemode 0 @a[tag=game]`);
                        startCheckForWinner();
                        startGameTimer(duration);
                    }, 30);
                }, 30);
            }, 30);
        }, 100);
    }, 100);
}

// Проверка количества игроков
function checkPlayerCount() {
    return world.getPlayers().length;
}

// Проверка победителя
function startCheckForWinner() {
    checkWinnerInterval = system.runInterval(() => {
        const playersWithTag = world.getPlayers().filter(player => player.getTags().includes("game"));
        if (playersWithTag.length === 1) {
            const winner = playersWithTag[0].name;
            world.getDimension('overworld').runCommandAsync(`tellraw @a {"rawtext":[{"text":"§a${winner} won the game!"}]}`);
            stopGame();
        }
    }, 20);
}

// Остановка игры
function stopGame() {
    if (checkWinnerInterval) {
        system.clearRun(checkWinnerInterval);
        checkWinnerInterval = null;
    }
    stopGameTimer();
    world.getDimension('overworld').runCommandAsync(`tag @a remove deat`);
    world.getDimension('overworld').runCommandAsync(`kill @a`);
    world.getDimension('overworld').runCommandAsync(`kill @e[type=tnt]`);
    world.getDimension('overworld').runCommandAsync(`kill @e[type=item]`);
    world.getDimension('overworld').runCommandAsync(`gamemode 2 @a`);
    world.getDimension('overworld').runCommandAsync(`tag @a remove game`);
    world.getDimension('overworld').runCommandAsync(`clear @a`);
    world.getDimension('overworld').runCommandAsync(`give @a[tag=admin] compass 1`);
}

// Обработка команд чата
world.beforeEvents.chatSend.subscribe((eventData) => {
    const player = eventData.sender;
    const args = eventData.message.split(" ");

    if (player.getTags().includes("admin")) {
        if (args[0].charAt(0) === "!") {
            eventData.cancel = true;   
            if (args[0] === "!start") {
                const playerCount = checkPlayerCount();                
                if (playerCount >= 2 && playerCount <= 4) {
                    startGame(args[1] ? parseInt(args[1]) : gameDuration);
                } else {
                    player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"§cRequires 2-4 players to start the game"}]}`);
                }
            } else if (args[0] === "!stop") {
                stopGame();
                player.runCommandAsync(`tellraw @a {"rawtext":[{"text":"§cThe game has been stopped"}]}`);     
            }
        }
    }
});

// Обработка смерти игрока
world.afterEvents.entityDie.subscribe((eventData) => {
    const player = eventData.deadEntity;
    if (player && player.typeId === "minecraft:player" && player.getTags().includes("game")) {
        player.runCommandAsync(`tag @s add deat`);
        player.runCommandAsync(`tag @s remove game`);
        player.runCommandAsync(`gamemode spectator @s`);
        system.runTimeout(() => {
            player.runCommandAsync(`tp @s[tag=deat] 50 66 50`);
        }, 50);
    }
});

// Форматирование времени
function formatTime(seconds) {
    const minutes = Math.floor(seconds / 60);
    const remainingSeconds = seconds % 60;
    return `${minutes}:${remainingSeconds < 10 ? '0' : ''}${remainingSeconds}`;
}

// Остановка таймера игры
function stopGameTimer() {
    if (gameTimer) {
        system.clearRun(gameTimer);
        gameTimer = null;
    }
}

// Запуск таймера игры
function startGameTimer(duration) {
    let remainingTime = duration;
    gameTimer = system.runInterval(() => {
        remainingTime--;
        if (remainingTime <= 0) {
            world.getDimension('overworld').runCommandAsync(`tellraw @a {"rawtext":[{"text":"§cTime is over!"}]}`);
            showRemainingPlayers();
            stopGameTimer();
            stopGame();
        } else {
            const formattedTime = formatTime(remainingTime);
            world.getDimension('overworld').runCommandAsync(`titleraw @a actionbar {"rawtext":[{"text":"§aTime left: ${formattedTime}"}]}`);
        }
    }, 20);
}

// Показ оставшихся игроков
function showRemainingPlayers() {
    const remainingPlayers = world.getPlayers().filter(player => player.getTags().includes("game")).map(player => player.name).join(", ");
    world.getDimension('overworld').runCommandAsync(`tellraw @a {"rawtext":[{"text":"§aRemaining players: ${remainingPlayers}"}]}`);
}

// Обработка использования предметов
world.beforeEvents.itemUse.subscribe(data => {
    let player = data.source;
    if (data.itemStack.typeId === "minecraft:compass") {
        system.run(() => mainMenu(player));
    }
});

// Главное меню
function mainMenu(player) {
    const title = "§l§o§uGame Menu";
    const form = new ActionFormData()
        .title(title)
        .body(`§l§o§5Welcome §d${player.nameTag}§5! Creator: BlackNobody90`)
        .button(`§3Play Game`)
        .button(`§5Game Settings`)
        .button(`§cExit Menu`);

    form.show(player).then(r => {
        if (r.selection === 0) chooseGame(player);
        if (r.selection === 1) gameSettings(player);
    });
}

// Выбор игры
function chooseGame(player) {
    const title = "§l§o§uChoose Game";
    const form = new ActionFormData()
        .title(title)
        .body(`§l§o§5Choose a game to play:`)
        .button(`§l§3Miners' Battle!`)
        .button(`§l§3Coming soon...`)
        .button(`§l§3Coming soon...`)
        .button(`§cBack`);

    form.show(player).then(r => {
        if (r.selection === 0) {
            startGame(gameDuration);
        }
        if (r.selection === 1) {
        	
        }
        if (r.selection === 2) {
        	
        }
        if (r.selection === 3) mainMenu(player);
    });
}

// Настройки игры
function gameSettings(player) {
    const title = "§l§o§uGame Settings";
    const form = new ModalFormData()
        .title(title)
        .textField(`§l§o§5Set Game Duration (seconds):`, `§o§7Default is 480`, `480`);

    form.show(player).then(r => {
        const duration = parseInt(r.formValues[0]);
        if (!isNaN(duration) && duration > 0) {
            setGameDuration(duration);
            player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"§aGame duration set to ${duration} seconds"}]}`);
        } else {
            player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"§cInvalid duration. Please enter a positive number."}]}`);
        }
        mainMenu(player);
    });
}

// Установка продолжительности игры
function setGameDuration(duration) {
    gameDuration = duration;
}
